package MT::Plugin::CMSContext;
use strict;
use MT;
use MT::Plugin;
use MT::Tag;

our $VERSION = '0.3';

use base qw( MT::Plugin );

@MT::Plugin::CMSContext::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::CMSContext({
    name => 'CMSContext',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    author_name => 'Alfasado, Inc.',
    author_link => 'http://alfasado.net/',
    version     => $VERSION,
    settings => new MT::PluginSettings ([
        ['plugin_active', { Default => 1 }],
        ['tpl_directory', { Default => 'alt-tmpl' }],
    ]),
    blog_config_template => 'cmsctx_config.tmpl',
    l10n_class => 'CMSContext::L10N',
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tags => {
            block => {
                'CMSContext' => \&_cms_ctx,
            },
        },
   });
}

sub init_request {
    my $plugin = shift;
    my ( $app ) = @_;
    if ( $app->isa('MT::App::CMS') ) {  
        my $blog = $app->blog;
        if ( defined $blog ) {
            my $get_from = 'blog:'. $blog->id;
            my $active = $plugin->get_config_value('plugin_active', $get_from);
            if ( $active ) {
                my $tpl_directory = $plugin->get_config_value('tpl_directory', $get_from);
                my $plugin_tmpdir = File::Spec->catdir( $plugin->path, 'tmpl', $tpl_directory );
                $app->config( 'AltTemplatePath', $plugin_tmpdir );
            }
        }
    }
}

sub _cms_ctx {
    my ($ctx, $args, $cond) = @_;
    my $app = MT->instance();
    my $author = $app->user;
    my $blog = $app->blog;
    my $q = $app->param;
    my $type = $q->param('_type');
    my $mode = $q->param('__mode');
    my $reedit = $q->param('reedit');
    my $id = $q->param('id');
    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    local $ctx->{__stash}{'blog'} = $blog;
    local $ctx->{__stash}{'blog_id'} = $blog->id;
    my $entry = MT::Entry->load( $id ) if ( ( $id ) && ( $mode eq 'view' ) 
                                                    && ( ( $type eq 'entry') || ( $type eq 'page') ) );
    $entry = MT::Entry->new if ( (! $id ) && ( $mode eq 'view' ) 
                                          && ( ( $type eq 'entry') || ( $type eq 'page') ) );
    if ( defined $entry ) {
        if ( $reedit ) {
            my @clumns = $entry->column_names;
            for my $key ($q->param) {
                if ( grep( /^$key$/, @clumns) ) {
                    $entry->$key( $q->param ( $key ) );
                } else {
                    if ( $key eq 'tags' ) {
                        my $tag_delim = chr( $author->entry_prefs->{ tag_delim } );
                        my @tags = MT::Tag->split( $tag_delim, $q->param ( $key ) );
                        $entry->set_tags(@tags);
                    }
                }
            }
        }
    }
    local $ctx->{__stash}{'entry'} = $entry if ( defined $entry );
    local $ctx->{__stash}{'category'} = MT::Category->load( $id ) if ( ( $id ) && ( $mode eq 'view' ) 
                                                                               && ( $type eq 'category' ) );
    my $out = $builder->build($ctx, $tokens, $cond);
    $out;
}

1;