﻿package MT::Plugin::PressRelease;
use strict;
use MT;
use MT::Plugin;

our $VERSION = '0.1';
our $SCHEMA_VERSION = '0.1';

use base qw( MT::Plugin );

@MT::Plugin::PressRelease::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::PressRelease({
    name => 'PressRelease',
    description => 'Manage Japanese and English Press Release on MT',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    'version' => $VERSION,
    'schema_version' => $SCHEMA_VERSION,
    settings => new MT::PluginSettings([
        ['directory', { Default => 'pdfs' }],
    ]),
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'blog' => {
                'name_en' => 'text(255)',
            },
            'entry' => {
                'excerpt_en' => 'text(255)',
                'pdf_ja' => 'text(255)',
                'pdf_en' => 'text(255)',
            },
        },
        callbacks => {
            'MT::App::CMS::template_param.edit_entry'
                => \&_entry_param,
            'cms_post_save.entry'
                => \&_save_pdfs,
            'cms_post_save.page'
                => \&_save_pdfs,
            'cms_pre_preview'
                => \&_preview_entry,
            'MT::App::CMS::template_output.list_entry'
                => \&_list_entry_output,
        },
        tags => {
            block => {
                'IfEntryPDFja?' => \&_if_entry_pdf_ja,
                'IfEntryPDFen?' => \&_if_entry_pdf_en,
            },
            function => {
                'EntryExcerptEn' => \&_entry_excerpt_en,
                'BlogNameEn' => \&_blog_name_en,
                'EntryPDFja' => \&_entry_pdf_ja,
                'EntryPDFen' => \&_entry_pdf_en,
            },
            modifier => {
                'path2url' => \&_path2url,
                'basename' => \&_basename,
            },
        }
   });
}

sub _preview_entry {
    my ($cb, $app, $obj, $data) = @_;
    my $q = $app->param;
    my $pdf_ja = $q->param('pdf_ja_org');
    my $pdf_en = $q->param('pdf_en_org');
    if ( $pdf_ja ) {
        push @$data,
          {
              data_name  => 'pdf_ja_org',
              data_value => $pdf_ja
          };
        $obj->pdf_ja($pdf_ja);
    }
    if ( $pdf_en ) {
        push @$data,
          {
              data_name  => 'pdf_en_org',
              data_value => $pdf_en
          };
        $obj->pdf_en($pdf_en);
    }
}

sub _path2url {
	my($text, $arg, $ctx) = @_;
	my $blog = $ctx->stash('blog');
    my $site_path = _site_path($blog);
    my $path = $text;
    $path =~ s/%r/$site_path/;
    if ( -f $path ) {
        my $site_url = _site_url($blog);
        $text =~ s/%r/$site_url/;
        return $text;
    } else {
        return '';
    }
}

sub _basename {
	my($text, $arg, $ctx) = @_;
    $text = File::Basename::basename($text);
    return $text;
}

sub _if_entry_pdf_ja {
    my ($ctx, $args, $cond) = @_;
    my $entry = $ctx->stash('entry')
                    or return $ctx->_no_entry_error('IfEntryPDFja');
    my $pdf_ja = $entry->pdf_ja;
    if ( $pdf_ja ) {
        my $blog = $ctx->stash('blog');
        my $site_path = _site_path( $blog );
        $pdf_ja =~ s/%r/$site_path/;
        if ( -f $pdf_ja ) {
            return 1;
        }
    }
    return 0;
}

sub _if_entry_pdf_en {
    my ($ctx, $args, $cond) = @_;
    my $entry = $ctx->stash('entry')
                    or return $ctx->_no_entry_error('IfEntryPDFen');
    my $pdf_en = $entry->pdf_en;
    if ( $pdf_en ) {
        my $blog = $ctx->stash('blog');
        my $site_path = _site_path( $blog );
        $pdf_en =~ s/%r/$site_path/;
        if ( -f $pdf_en ) {
            return 1;
        }
    }
    return 0;
}

sub _blog_name_en {
    my ($ctx, $args, $cond) = @_;
    my $blog = $ctx->stash('blog');
    return defined( $blog->name_en ) ? $blog->name_en : '';
}

sub _entry_excerpt_en {
    my ($ctx, $args, $cond) = @_;
    my $entry = $ctx->stash('entry');
    return defined( $entry->excerpt_en ) ? $entry->excerpt_en : '';
}

sub _entry_pdf_ja {
    my ($ctx, $args, $cond) = @_;
    my $entry = $ctx->stash('entry')
                    or return $ctx->_no_entry_error('EntryPDFja');
    my $pdf_ja = $entry->pdf_ja;
    if ( $pdf_ja ) {
        my $blog = $ctx->stash('blog');
        my $site_path = _site_path( $blog );
        my $pdf_ja_path = $pdf_ja;
        $pdf_ja_path =~ s/%r/$site_path/;
        if ( -f $pdf_ja_path ) {
            my $site_url = _site_url( $blog );
            $pdf_ja =~ s/%r/$site_url/;
            return $pdf_ja;
        }
    }
    return '';
}

sub _entry_pdf_en {
    my ($ctx, $args, $cond) = @_;
    my $entry = $ctx->stash('entry')
                    or return $ctx->_no_entry_error('EntryPDFen');
    my $pdf_en = $entry->pdf_en;
    if ( $pdf_en ) {
        my $blog = $ctx->stash('blog');
        my $site_path = _site_path( $blog );
        my $pdf_en_path = $pdf_en;
        $pdf_en_path =~ s/%r/$site_path/;
        if ( -f $pdf_en_path ) {
            my $site_url = _site_url( $blog );
            $pdf_en =~ s/%r/$site_url/;
            return $pdf_en;
        }
    }
    return '';
}

sub _entry_param {
    my ($cb, $app, $param, $tmpl) = @_;
    my $q = $app->param;
    my $blog = $app->blog;
    my $blog_id = $q->param('blog_id');
    my $reedit = $q->param('reedit');
    my $pdf_ja;
    my $pdf_en;
    if ($reedit) {
        $pdf_ja = $q->param('pdf_ja_org');
        $param->{'pdf_ja'} = $pdf_ja;
        $pdf_en = $q->param('pdf_en_org');
        $param->{'pdf_en'} = $pdf_en;
    } else {
        $pdf_ja = $param->{'pdf_ja'};
        $pdf_en = $param->{'pdf_en'};
    }

    my $site_path = _site_path($blog);
    if ( $pdf_ja ) {
        $pdf_ja =~ s/%r/$site_path/;
        unless ( -f $pdf_ja ) {
            $param->{'pdf_ja'} = 0;
        }
        $pdf_en =~ s/%r/$site_path/;
    }
    if ( $pdf_en ) {
        unless ( -f $pdf_en ) {
            $param->{'pdf_en'} = 0;
        }
    }
#     例えば 初期値をテンプレートモジュール化してインクルードする等のアイデア
#     my $init_ja_module = MT::Template->load( { blog_id => $blog_id, name => '問合せ先 (日本語)' } );
#     if ( defined $init_ja_module ) {
#         $param->{'init_ja_module'} = 1;
#     }
#     my $init_en_module = MT::Template->load( { blog_id => $blog_id, name => '問合せ先 (英語)' } );
#     if ( defined $init_en_module ) {
#         $param->{'init_en_module'} = 1;
#     }
}

sub _save_pdfs {
    my ($eh, $app, $obj, $original) = @_;
    my $q = $app->param;
    my $return_args = $q->param('return_args');
    if ($return_args =~ /&is_power_edit=1/) {
        return;
    }
    my $blog = $app->blog;
    my $fmgr = $blog->file_mgr;
    my $site_path = _site_path($blog);
    my $directory = $plugin->get_config_value('directory');
    my $save_obj;
    my $label = 'release' . sprintf( '%04d', $obj->id ) . '_';
    for ( 1..2 ) {
        my $lang = 'ja';
        if ($_ == 2) {
            $lang = 'en';
        }
        my $file = $q->upload( 'pdf_' . $lang );
        if ( $file ) {
            if ( $file =~ /\.pdf$/ ) {
                my $out = File::Spec->catfile ( $site_path, $directory, $label . $lang . '.pdf' );
                my $dir = File::Basename::dirname($out);
                $dir =~ s!/$!! unless $dir eq '/';
                unless ($fmgr->exists($dir)) {
                    $fmgr->mkpath($dir);
                }
                open (OUT, ">$out") or die "Can't open $out!";
                binmode (OUT);
                while( read ($file, my $buffer, 1024) ) {
                    print OUT $buffer;
                }
                close (OUT);
                $out =~ s/$site_path/%r/;
                if ( $lang eq 'ja' ) {
                    $obj->pdf_ja( $out );
                } else {
                    $obj->pdf_en( $out );
                }
                $save_obj = 1;
            }
        } else {
            my $pdf_org = $q->param( 'pdf_' . $lang . '_org' );
            my $remove = $q->param( 'pdf_' . $lang . '_remove' );
            if ( $pdf_org ) {
                if ( $remove ) {
                    $pdf_org =~ s/%r/$site_path/;
                    if ( -f $pdf_org ) {
                        unlink $pdf_org;
                    }
                }
                if ( $lang eq 'ja' ) {
                    if ( $remove ) {
                        $obj->pdf_ja('');
                    } else {
                        $obj->pdf_ja( $pdf_org );
                    }
                } else {
                    if ( $remove ) {
                        $obj->pdf_en('');
                    } else {
                        $obj->pdf_en( $pdf_org );
                    }
                }
                $save_obj = 1;
            }
        }
    }
    if ( $save_obj ) {    
        $obj->save
            or return $app->trans_error('Error saving entry: [_1]', $obj->errstr);
    }
}

sub _list_entry_output {
    my ( $cb, $app, $tmpl ) = @_;
    my $begin = quotemeta('<td class="view si status-view">');
    my $end = quotemeta('</td>');
    $$tmpl =~ s/($begin)(.*?)($end)/'<td class="view si status-view" nowrap>'.$2._add_en_link($2).$3/esg;
}

sub _ja2en {
    my $text = shift;
    $text =~ s/_ja$/_en/;
    return $text;
}

sub _add_en_link {
    my $src = shift;
    $src =~ s/_ja\.html/_en.html/;
    return $src;
}

sub _site_path {
    my $blog = shift;
    my $site_path = $blog->site_path;
    if ( $site_path =~ /(.*)\/$/ ) {
        $site_path = $1;
    }
    if ( $^O eq 'MSWin32' ) {
        if ( $site_path =~ /(.*)\\$/ ) {
            $site_path = $1;
        }
    }
    return $site_path;
}

sub _site_url {
    my $blog = shift;
    my $site_url = $blog->site_url;
    if ( $site_url =~ /(.*)\/$/ ) {
        $site_url = $1;
    }
    return $site_url;
}

1;